@@ -47,8 +47,14 @@ enum certificate_type_t {
 	CERT_X509_AC,
 	/** trusted, preinstalled public key */
 	CERT_TRUSTED_PUBKEY,
+	/** PKCS#10 certificate request */
+	CERT_PKCS10_REQUEST,
 	/** PGP certificate */
-	CERT_PGP,
+	CERT_GPG,
+	/** Pluto cert_t (not a certificate_t), either x509 or PGP */
+	CERT_PLUTO_CERT,
+	/** Pluto x509crl_t (not a certificate_t), certificate revocation list */
+	CERT_PLUTO_CRL,
 };
 
 /**
@@ -82,7 +88,7 @@ extern enum_name_t *cert_validation_names;
 /**
  * An abstract certificate.
  *
- * A certificate designs a subject-issuer relationship. It may have an 
+ * A certificate designs a subject-issuer relationship. It may have an
  * associated public key.
  */
 struct certificate_t {
@@ -90,7 +96,7 @@ struct certificate_t {
 	/**
 	 * Get the type of the certificate.
 	 *
-	 * @return			certifcate type
+	 * @return			certificate type
 	 */
 	certificate_type_t (*get_type)(certificate_t *this);
 
@@ -100,7 +106,7 @@ struct certificate_t {
 	 * @return			subject identity
 	 */
 	identification_t* (*get_subject)(certificate_t *this);
-	
+
 	/**
 	 * Check if certificate contains a subject ID.
 	 *
@@ -111,14 +117,14 @@ struct certificate_t {
 	 * @return 			matching value of best match
 	 */
 	id_match_t (*has_subject)(certificate_t *this, identification_t *subject);
-		
+
 	/**
 	 * Get the issuer which signed this certificate.
 	 *
 	 * @return			issuer identity
 	 */
 	identification_t* (*get_issuer)(certificate_t *this);
-	
+
 	/**
 	 * Check if certificate contains an issuer ID.
 	 *
@@ -129,7 +135,7 @@ struct certificate_t {
 	 * @return 			matching value of best match
 	 */
 	id_match_t (*has_issuer)(certificate_t *this, identification_t *issuer);
-	
+
 	/**
 	 * Check if this certificate is issued and signed by a specific issuer.
 	 *
@@ -137,14 +143,14 @@ struct certificate_t {
 	 * @return 			TRUE if certificate issued by issuer and trusted
 	 */
 	bool (*issued_by)(certificate_t *this, certificate_t *issuer);
-	
+
 	/**
 	 * Get the public key associated to this certificate.
 	 *
 	 * @return			newly referenced public_key, NULL if none available
 	 */
 	public_key_t* (*get_public_key)(certificate_t *this);
-	
+
 	/**
 	 * Check the lifetime of the certificate.
 	 *
@@ -155,21 +161,21 @@ struct certificate_t {
 	 */
 	bool (*get_validity)(certificate_t *this, time_t *when,
 						 time_t *not_before, time_t *not_after);
-	
+
 	/**
 	 * Is this newer than that?
 	 *
 	 * @return			TRUE if newer, FALSE otherwise
 	 */
 	bool (*is_newer)(certificate_t *this, certificate_t *that);
-	
+
 	/**
 	 * Get the certificate in an encoded form.
 	 *
 	 * @return				allocated chunk of encoded cert
 	 */
 	chunk_t (*get_encoding)(certificate_t *this);
-	
+
 	/**
 	 * Check if two certificates are equal.
 	 *
@@ -177,18 +183,18 @@ struct certificate_t {
 	 * @return				TRUE if certificates are equal
 	 */
 	bool (*equals)(certificate_t *this, certificate_t *other);
-	
+
 	/**
 	 * Get a new reference to the certificate.
 	 *
-	 * @return			this, with an increased refcount 
+	 * @return			this, with an increased refcount
 	 */
 	certificate_t* (*get_ref)(certificate_t *this);
-	
+
 	/**
-     * Destroy a certificate.
-     */
-    void (*destroy)(certificate_t *this);
+	 * Destroy a certificate.
+	 */
+	void (*destroy)(certificate_t *this);
 };
 
 #endif /** CERTIFICATE_H_ @}*/
