@@ -18,14 +18,16 @@
 
 #include <stdlib.h>
 
-#include <utils.h>
-#include <chunk.h>
-#include <utils/identification.h>
-#include <utils/host.h>
+#include "debug.h"
+#include "threading/thread.h"
+#include "utils/identification.h"
+#include "utils/host.h"
 #ifdef LEAK_DETECTIVE
-#include <utils/leak_detective.h>
+#include "utils/leak_detective.h"
 #endif
 
+#define CHECKSUM_LIBRARY IPSEC_DIR"/libchecksum.so"
+
 typedef struct private_library_t private_library_t;
 
 /**
@@ -61,17 +63,25 @@ void library_deinit()
 	this->public.plugins->destroy(this->public.plugins);
 	this->public.settings->destroy(this->public.settings);
 	this->public.creds->destroy(this->public.creds);
+	this->public.encoding->destroy(this->public.encoding);
 	this->public.crypto->destroy(this->public.crypto);
 	this->public.fetcher->destroy(this->public.fetcher);
 	this->public.db->destroy(this->public.db);
 	this->public.printf_hook->destroy(this->public.printf_hook);
-	
+	if (this->public.integrity)
+	{
+		this->public.integrity->destroy(this->public.integrity);
+	}
+
 #ifdef LEAK_DETECTIVE
 	if (this->detective)
 	{
 		this->detective->destroy(this->detective);
 	}
 #endif /* LEAK_DETECTIVE */
+
+	threads_deinit();
+
 	free(this);
 	lib = NULL;
 }
@@ -79,21 +89,23 @@ void library_deinit()
 /*
  * see header file
  */
-void library_init(char *settings)
+bool library_init(char *settings)
 {
 	printf_hook_t *pfh;
 	private_library_t *this = malloc_thing(private_library_t);
 	lib = &this->public;
-	
+
+	threads_init();
+
 	lib->leak_detective = FALSE;
-	
+
 #ifdef LEAK_DETECTIVE
 	this->detective = leak_detective_create();
 #endif /* LEAK_DETECTIVE */
 
 	pfh = printf_hook_create();
 	this->public.printf_hook = pfh;
-	
+
 	pfh->add_handler(pfh, 'b', mem_printf_hook,
 					 PRINTF_HOOK_ARGTYPE_POINTER, PRINTF_HOOK_ARGTYPE_INT,
 					 PRINTF_HOOK_ARGTYPE_END);
@@ -112,12 +124,31 @@ void library_init(char *settings)
 					 PRINTF_HOOK_ARGTYPE_END);
 	pfh->add_handler(pfh, 'Y', identification_printf_hook,
 					 PRINTF_HOOK_ARGTYPE_POINTER, PRINTF_HOOK_ARGTYPE_END);
-	
+
 	this->public.settings = settings_create(settings);
 	this->public.crypto = crypto_factory_create();
 	this->public.creds = credential_factory_create();
+	this->public.encoding = key_encoding_create();
 	this->public.fetcher = fetcher_manager_create();
 	this->public.db = database_factory_create();
 	this->public.plugins = plugin_loader_create();
+	this->public.integrity = NULL;
+
+	if (lib->settings->get_bool(lib->settings,
+								"libstrongswan.integrity_test", FALSE))
+	{
+#ifdef INTEGRITY_TEST
+		this->public.integrity = integrity_checker_create(CHECKSUM_LIBRARY);
+		if (!lib->integrity->check(lib->integrity, "libstrongswan", library_init))
+		{
+			DBG1(DBG_LIB, "integrity check of libstrongswan failed");
+			return FALSE;
+		}
+#else /* !INTEGRITY_TEST */
+		DBG1(DBG_LIB, "integrity test enabled, but not supported");
+		return FALSE;
+#endif /* INTEGRITY_TEST */
+	}
+	return TRUE;
 }
 
