@@ -66,7 +66,7 @@ typedef enum {
 	KW_TYPE,
 	KW_PFS,
 	KW_COMPRESS,
-		KW_INSTALLPOLICY,
+	KW_INSTALLPOLICY,
 	KW_AUTH,
 	KW_AUTHBY,
 	KW_EAP,
@@ -76,6 +76,10 @@ typedef enum {
 	KW_IKELIFETIME,
 	KW_KEYLIFE,
 	KW_REKEYMARGIN,
+	KW_LIFEBYTES,
+	KW_MARGINBYTES,
+	KW_LIFEPACKETS,
+	KW_MARGINPACKETS,
 	KW_KEYINGTRIES,
 	KW_REKEYFUZZ,
 	KW_REKEY,
@@ -86,6 +90,7 @@ typedef enum {
 	KW_DPDDELAY,
 	KW_DPDTIMEOUT,
 	KW_DPDACTION,
+	KW_INACTIVITY,
 	KW_MODECONFIG,
 	KW_XAUTH,
 	KW_MEDIATION,
@@ -113,6 +118,7 @@ typedef enum {
    /* end keywords */
 	KW_HOST,
 	KW_NEXTHOP,
+	KW_IKEPORT,
 	KW_SUBNET,
 	KW_SUBNETWITHIN,
 	KW_PROTOPORT,
@@ -122,11 +128,16 @@ typedef enum {
 	KW_HOSTACCESS,
 	KW_ALLOWANY,
 	KW_UPDOWN,
+	KW_AUTH1,
+	KW_AUTH2,
 	KW_ID,
+	KW_ID2,
 	KW_RSASIGKEY,
 	KW_CERT,
+	KW_CERT2,
 	KW_SENDCERT,
 	KW_CA,
+	KW_CA2,
 	KW_GROUPS,
 	KW_IFACE,
 
@@ -135,6 +146,7 @@ typedef enum {
 
    /* left end keywords */
 	KW_LEFT,
+	KW_LEFTIKEPORT,
 	KW_LEFTNEXTHOP,
 	KW_LEFTSUBNET,
 	KW_LEFTSUBNETWITHIN,
@@ -162,6 +174,7 @@ typedef enum {
 
    /* right end keywords */
 	KW_RIGHT,
+	KW_RIGHTIKEPORT,
 	KW_RIGHTNEXTHOP,
 	KW_RIGHTSUBNET,
 	KW_RIGHTSUBNETWITHIN,
