@@ -183,6 +183,13 @@ CREATE TABLE leases (
   released INTEGER NOT NULL
 );
 
+DROP TABLE IF EXISTS attributes;
+CREATE TABLE attributes (
+  id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
+  type INTEGER NOT NULL,
+  value BLOB NOT NULL
+);
+
 DROP TABLE IF EXISTS ike_sas;
 CREATE TABLE ike_sas (
   local_spi BLOB NOT NULL PRIMARY KEY,
@@ -190,9 +197,9 @@ CREATE TABLE ike_sas (
   id INTEGER NOT NULL,
   initiator INTEGER NOT NULL,
   local_id_type INTEGER NOT NULL,
-  local_id_data BLOB NOT NULL,
+  local_id_data BLOB DEFAULT NULL,
   remote_id_type INTEGER NOT NULL,
-  remote_id_data BLOB NOT NULL,
+  remote_id_data BLOB DEFAULT NULL,
   host_family INTEGER NOT NULL,
   local_host_data BLOB NOT NULL,
   remote_host_data BLOB NOT NULL,
